/* Copyright © 2017-2020 ABBYY Production LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
--------------------------------------------------------------------------------------------------------------*/

#pragma once

#include "../Node.h"

namespace NeoOnnx {

// Conv operator graph node
class CConvNode : public COpNode {
public:
	CConvNode( int nodeIndex, const onnx::NodeProto& conv, int opsetVersion );

	// CNode methods' realizations
	void CalcOutputTensors( CTensorCache& tensors, IMathEngine& mathEngine ) override;
	void LabelTensorDims( const CTensorCache& tensors, CDimCache& dims ) override;
	void AddLayers( const CGraph& graph, const CTensorCache& tensors, const CDimCache& dims,
		CNeoMLLinkCache& neoMLLinks, CDnn& dnn ) override;

private:
	const int group; // groups count
	const CString autoPad; // padding mode
	CFastArray<int, 8> strides; // kernel strides
	CFastArray<int, 8> pads; // convolution paddings
	CFastArray<int, 8> dilations; // convolution dilations

	void add2dConvLayer( const CTensorCache& tensors, CNeoMLLinkCache& neoMLLinks, CDnn& dnn );
	void add3dConvLayer( const CTensorCache& tensors, CNeoMLLinkCache& neoMLLinks, CDnn& dnn );
};

} // namespace NeoOnnx
